#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <strings.h>
#include <assert.h>
#include "query.h"

time_t loctime;
struct tm timeinfo, *loctimeinfo;

char *strMonth[] = {
  "January", "February", "March", "April", "May", "June", "July",
  "August", "September", "October", "November", "December"
};

char *strDay[] = {
  "Sunday", "Monday", "Tuesday", "Wednesday", "Thursday",
  "Friday", "Saturday"
};

int getYear(char *querystring) {
  /* return year 1.., 0 on error */
  int year;
  char yearStr[255];

  /* printf("\ninput year: "); */
  /* scanf("%d", &year); */

  year = get_var(yearStr, "year", querystring);
  
  if (year == EOF) {
    year = 0;
  } else {
    year = atoi(yearStr);
    if (year<1) {
      year = 0;
    }
  }
  return year;
}

int getDay(char *querystring) {
  /* 
     returns day 1.., 0 on error.
     if day exceeds max for a month it will wrap to the next month.
  */
  int day;
  char dayStr[255];

  /* printf("\ninput day: "); */
  /* scanf("%d", &day); */

  day = get_var(dayStr, "day", querystring);
  
  if (day == EOF) {
    day = 0;
  } else {
    day = atoi(dayStr);
    if (day<1) {
      day = 0;
    }
  }
  return day;
}

int getMonth(char *querystring) {
  /* returns month 1..12, 0 on error. */
  int month;
  char monthStr[255];
  
  /* printf("\ninput month: "); */
  /* scanf("%d", &month); */

  month = get_var(monthStr, "month", querystring);
  
  if (month == EOF) {
    month = 0;
  } else {
    month = atoi(monthStr);
    if ((month<1)||(month>12)) {
      month = 0;
    }
  }
  return month;
}

int main(int argc, char **argv) {
  int year, day, month, week;
  char queryStr[2048];
  char weekStr[64];
  
  strcpy(queryStr, getenv("QUERY_STRING"));

  bzero(&timeinfo, sizeof(struct tm));
  timeinfo.tm_isdst = -1;
  timeinfo.tm_mon = 0;
 
  printf(
	 "\n<h1>Day of the Year</h1>"
	 "\n<div>"
	 "\n<img src=\"/images/calendar.png\" />"
	 "\n</div>\n"
	 );

  year = getYear(queryStr);
  month = getMonth(queryStr);
  day = getDay(queryStr);

  /* printf("Enter month day year: ");	     */
  /* scanf("%d %d %d", &month, &day, &year); */

  
  if (day && month && year) {
    assert(month>0);
    assert(month<13);
    
    timeinfo.tm_mon  = month-1;  
    timeinfo.tm_mday = day;
    timeinfo.tm_year = year - 1900;

    loctime = mktime (&timeinfo);
    loctimeinfo = localtime (&loctime);
    if (strftime(weekStr, sizeof weekStr, "%W", &timeinfo))
      week = atoi(weekStr);
    else
      week = -1;

    printf("\n<p>");
    printf("\nThe date you input was %s %d, %d.",
	   strMonth[month-1], day, year
	   );
    printf("\n</p>");
    printf("\n<p>");
    printf(
	   "\n%9s, %9s %2d, %4d is day %3d of %4d.  ",
	   strDay[timeinfo.tm_wday],
	   strMonth[timeinfo.tm_mon], timeinfo.tm_mday,
	   timeinfo.tm_year+1900,     timeinfo.tm_yday+1,
	   timeinfo.tm_year+1900
	   );

    if (week==-1) {
      printf("\n</p><p>Error in getting week number.</p>");
    } else {
      if (week==0) { 
	week=53;
	year--;
      }
      printf(
	     "\nWhich is week %d of %d.\n", week, year
	     );
    }
  } else {
    printf(
	   "\n<p>Error: Month, day and year should be positive integers.</p>"
	   "\n<p>Month must be in range 1..12."
	   );
  }
  printf("\n</p>\n<p>"
	 "\n<a href=\"/dayofyear.html\">Return to Day of the Year</a></p>"
	 "\n<hr />\n"
	 );
  return EXIT_SUCCESS;
}

